// Bibliotheken zum Einbinden (Zufallszahlen, 
//                             Ein-Ausgabe,Mathematik)
#include <cstdlib>
#include <stdio.h>
#include <math.h>
#include "hilf.h"
#include <iostream>
using namespace std;

void zufall(int n,float *x) 
// n zufallszahlen auf (0,1)
// Parameter seed.. Zeit in sec seit 1.1.1970 
//                                  (im 1. Schritt)
//     fuer zufaellige Initialisierung der Zufallszahlen    
//     Typ ist static int, d.h. bei erneutem Aufruf 
//     wird diese Variable auf den alten Wert gesetzt.
              
{
  unsigned long  m,z1,z,k;
  static long  seed=time(0);
  static long  z0=1;
   
  z1=seed;
  m=((1<<30)-1)*2+1;
  for (k=1;k<n+1;k++)
    {
      z=(1999*z1+4444*z0)%m; // Generation gemaess Vorlesung
      z0=z1;
      z1=z;  
      x[k-1]=float(z)/float(m); // Transformation auf [0,1)
    }
   seed=z1;
}

void Neu(int *j,int *y,int k)
// Input: y die bereits gezogenen Kugeln
//        j .. sei die j-groesste nichtgezogene Kugel
//        k .. die Anzahl bereits gezogender Kugeln

// Output: Bestimme daraus die Nr. der aktuellen Kugel und 
// arbeite sie in y ein
{
  int l,i;
  i=1;
  l=k;
  while (i<k)
    {
      if (*j>=y[i-1])
        { *j=*j+1; // Streichen der schon gezogenen Kugeln
          i=i+1;}
      else // sonst der Kugeln in richtiger reihenfolge
        { 
          for (l=k-1;l>=i;l--)
	    {
	      y[l]=y[l-1];
            }
          l=l+1;
          i=k;
        }  
    }
  y[l-1]=*j;
  
}

void Ziehen(int n, int m,int *y)
// Ziehen von n aus m Kugeln:
// Input: n,m
// Output: y[n], die gezogenen Kugeln sortiert
{
  int j,k,l;
  const int N=49000;
  float x[N];
  zufall(n,x);
  l=m;
  // Simulation Ziehen ohne Zuruecklegen, jedes Mal eine
  // Kugel (l) weniger
  for (k=1;k<n+1;k++)
    {
      j=1+int(floor(l)*x[k-1]);
      Neu(&j,y,k);   
      l=l-1;
    }   
}

void vergleich(int *x,int *y,int *j,int n)
// Wieviele Zahlen in x und y sind gleich? 
// Input: x,y, Integer Felder der Laenge n, aufsteigend sortiert
// Output: j Anzahl der gleichen Zahlen
// Algorithmus ist aehnlich zu den Mergesort-Algorithmen
{
  int r,s;
  *j=0;
  r=0;
  s=0;
  while (max(r,s)<n)
    {
      if (x[r]<y[s])
        {
        r=r+1;
        }
      else
        {
	  if (x[r]>y[s]) 
            {
            s=s+1;
            }
          else
            {
            r=r+1;
	    s=s+1; 
            *j=*j+1;
            }
        }
    }
}

int main()
{ const int K=49000; // Maximal aus 49000 Kugeln auswaehlen
  int y[K],x[K],r[K];
  int l,L1,k,M,N,j;
  // Parameter
  // x.. Gewuenschte Zahlen
  // y.. Gezogene Zahlen
  // r.. Zufallsgroesse der hypergeometrischen Verteilung H(n,N,M) mit n=M

  // Weiterhin: l.. Anzahl Ziehungen
  //            N.. Anzahl Kugeln
  //            M.. Anzahl zu ziehender Kugeln

  // 1. Schritt: Eingabe aller Groessen
  cout<< "N, M eingeben:";
  cin >> N>> M;
  cout<<" Anzahl Versuche? " << endl;
  cin >> l;
  for (k=1;k<M+1;k++)
    {
      cout << k<<"-te Zahl eingeben" << endl;
      cin >> x[k-1];
      r[k-1]=0;
    }
  // 2. Schritt: Ziehen der n Zahlen und Vergleich 
  // mit den gewuenschten Zahlen
  r[M]=0;
  for (L1=0;L1<l;L1++)
    {
      Ziehen(M,N,y);
      vergleich(x,y,&j,M); 
      if (j>M-3)  // Ausgabe guter Tips
        {
	  cout <<j<<" Treffer im "<<L1+1<<"ten Versuch: Die Zahlen lauten ";
        ausgabeint(y,M);
        } 
     r[j]=r[j]+1;
    }

  // 3. Schritt :Ausgabe der Verteilung
  cout << "Die Verteilung ist 0/1/.../" <<M<< endl;
  ausgabeint(r,M+1);  
  return 0;
}
